/* 
 * Purpose: Construct the estimates for life expectancy lost for each disease
		from GBD, using the cohort life table method. For each disease, we 
		compare actual (baseline) life expectancy with the counterfactual 
		no-disease life expectancy (from lowering the mortality rate). 
		The difference is the contribution of that disease to life expectancy lost.
 * Methodology: https://www.statsdirect.com/help/survival_analysis/abridged_life_table.htm
 */
 

* Change to your directory
local ROOT "C:\Users\Claire Fan\Desktop\AQLI_GBD_Comparisons"

local region = "global" //options are global, china, india

/*************** Part 1: Set up the actual (baseline) life table **************/
tempfile actual_life_table
insheet using "`ROOT'/WHO_`region'_lt.csv", names clear
drop indicator
replace agegroup = subinstr(agegroup, "years", "", .)
replace agegroup = subinstr(agegroup, "year", "", .)
replace agegroup = subinstr(agegroup, " ", "", .)

/* Age categories:
		Since the WHO's data on global mortality rates ends at 85+, but the GBD
		categories either end at 80+ or 95+, we decided to truncate the WHO
		at 80+. This leads to a slightly different estimate of life expectancy 
		at birth from the WHO table, recalculated here, but leads to consistent 
		comparisons across diseases from GBD. To recalculate with the 80+ as the
		max category, it is sufficient to just have the lx, nLx, ndx, and nqx 
		variables. These are, for an artificial cohort of 100k births:
			lx = number of life-years remaining at the beginning of the age group
			nLx = life-years lived within the age group
			ndx = deaths within the age group
			nqx = probability that individual will die in the interval
			nMx = annual death rate, as a probability, in the interval */
keep if inlist(lt_var, "lx", "nLx", "ndx", "nqx", "nMx")
* Generate the age category variable
gen age_n = substr(agegroup, 1, strpos(agegroup, "-") - 1)
destring age_n, replace
gen age_cat = .
replace age_cat = 1 if agegroup == "<1"
replace age_cat = 2 if agegroup == "1-4"
replace age_cat = (age_n/5) + 2 if age_cat == . & agegroup != "85+"
qui summ age_cat
replace age_cat = `r(max)' + 1 if agegroup == "85+"
drop age_n
* Prepare for the reshape
rename male _male
rename female _female
reshape wide @_male @_female, i(agegroup age_cat) j(lt_var) string
sort age_cat
* Convert from separate 80-84 and 85+ obs to just 80+ for comparison with GBD
local new_num_obs = _N + 1
set obs `new_num_obs'
replace agegroup = "80+" if _n == `new_num_obs'
* For 80+, the lx variable should have the value at age 80, i.e. from 80-84
foreach v in lx_male lx_female {
	qui summ `v' if agegroup == "80-84"
	replace `v' = `r(mean)' if agegroup == "80+"
}
* For 80+, the nqx value should be 1 because everyone in the 80+ group dies in that group.
foreach v in nqx_male nqx_female {
	replace `v' = 1 if agegroup == "80+"
}
* For 80+, the ndx and nLx values are the sum of the values from 80-84 and 85+
foreach v in ndx_male ndx_female nLx_male nLx_female {
	qui summ `v' if agegroup == "80-84"
	local val1 = `r(mean)'
	qui summ `v' if agegroup == "85+"
	replace `v' = `r(mean)' + `val1' if agegroup == "80+"
}
* For 80+, the nMx value is simply the ratio of ndx and nLx, its general definition
foreach s in "male" "female" {
	replace nMx_`s' = ndx_`s' / nLx_`s' if agegroup == "80+"
	qui summ nMx_`s' if agegroup == "80+"
	local terminal_nMx_`s' = `r(mean)'
}
* Now remove the rows for 80-84 and 85+
drop if inlist(agegroup, "80-84", "85+")
qui summ age_cat
local new_max_age_cat = `r(max)' + 1
replace age_cat = `new_max_age_cat' if agegroup == "80+"
gen agegap = 5 // def'd to be number of yrs, inclusive, in the age category
replace agegap = 1 if age_cat == 1
replace agegap = 4 if age_cat == 2
replace agegap = . if age_cat == `new_max_age_cat'

/* Get precise values:
		Although life table provides nqx and nMx, there's a lot of rounding, which leads
		to inaccuracies when using nqx later on. For a more accurate value of nqx,
		back it out from lx and ndx using the identity nqx = ndx / lx */
foreach sex in "male" "female" {
	replace nqx_`sex' = ndx_`sex' / lx_`sex'
	/* Similarly, to cure the rounding issue, for all but the terminal age group,
			back out nMx from the identity 
			q = n * M / (1 + (1 - a) * n * M),
			where n is the agegap and a is defined below. For the terminal,
			M = l / L */
	/* Using the L_i formula of L = n * (l - d) + a * n * d, then solving for a */
	gen ax_`sex' = (nLx_`sex' - agegap * (lx_`sex' - ndx_`sex')) / (agegap * ndx_`sex')
	replace nMx_`sex' = nqx_`sex' / (agegap * (1 - nqx_`sex' * (1 - ax_`sex'))) if age_cat !=`new_max_age_cat'
	replace nMx_`sex' = lx_`sex' / nLx_`sex' if age_cat ==`new_max_age_cat'
	/* Now re-calculate the life expectancy at birth given the new truncated set of
			age groups */
	gen nTx_`sex'=nLx_`sex' if age_cat ==`new_max_age_cat'
	/* Recursively calculate the total life-years lived for people who survive
			to the beginning of the age group - this is how nTx is defined */
	local penultimate = `new_max_age_cat' - 1
	forvalues i=1/`penultimate' {
		replace nTx_`sex'=nLx_`sex'[_n] + nTx_`sex'[_n+1] if age_cat ==`new_max_age_cat'-`i'
	}
	* Now calculate the remaining life expectancy conditional on living to the age group 
	gen ex_`sex'=nTx_`sex'/lx_`sex'
	local ex_birth_`sex' = ex_`sex'[1] // life expectancy at birth
}
save `"`actual_life_table'"', replace
di "Global estimated life expectancy at birth, women: `ex_birth_female'"
di "Global estimated life expectancy at birth, men: `ex_birth_male'"
	
	
/****** Part 2: Determine the life expectancy were a disease not to exist *****/
/*		GBD gives "Rate per 100k", or "in a single country-year-age-sex, 
		the deaths due to cause X divided by the population." 
		http://www.healthdata.org/terms-defined
		
		For each disease, by shifting values of M by these rates, 
		we get what M would be in a world without that disease.
*/

tempfile gbd

* First, read in and clean the GBD death rates file
insheet using "`ROOT'/gbdrates_`region'.csv", names clear
keep sex age cause val
replace sex = lower(sex)
rename val shifter_ // mortality rate shifter
replace shifter_ = shifter_ / 100000 // to convert from rate to a probability
* Create a merge-able age category
gen age_n = substr(age, 1, strpos(age, "to") - 2)
destring age_n, replace
gen age_cat = .
replace age_cat = 1 if age == "<1 year"
replace age_cat = 2 if age == "1 to 4"
replace age_cat = (age_n/5) + 2 if age_cat == . & age != "80 plus"
qui summ age_cat
replace age_cat = `r(max)' + 1 if age == "80 plus"
drop age_n
reshape wide shifter_@, i(age cause age_cat) j(sex) string
save `"`gbd'"', replace

* Now loop over the causes and compute the counterfactual life expectancy
levelsof(cause), local(the_causes)
foreach c of local the_causes {
	use "`gbd'", clear
	di "Currently considered cause of death: `c'"
	keep if cause == `"`c'"'
	merge 1:1 age_cat using "`actual_life_table'", assert(match using) nogen
	sort age_cat
	/* For some diseases, GBD doesn't report mortality shifters for some age categories. 
	In those cases, assume the disease has 0 effect on mortality in that age category */
	foreach s in "male" "female" {
		replace shifter_`s' = 0 if shifter_`s' == .
	}
	/* Shift the mortality value for each sex - consider world without cause of death
		 This requires us to recalculate lx and ndx for each interval, not just the 
			last interval as in part 1. */
	drop nqx* lx* nLx* ndx* nTx* ex*
	
	foreach s in "male" "female" {
		replace nMx_`s' = nMx_`s' - shifter_`s'
		* Recalculate q, the age group probability of death
		gen nqx_`s' = agegap * nMx_`s' / (1 + (1 - ax_`s') * agegap * nMx_`s')
		replace nqx_`s' = 1 if age_cat == `new_max_age_cat'

		gen lx_`s' = 100000
		/* Deaths per interval */
		forvalues i=2/`new_max_age_cat' {
			replace lx_`s' = lx_`s'[_n-1]*(1 - nqx_`s'[_n-1]) if _n == `i'
		}
		gen ndx_`s' = nqx_`s' * lx_`s'
		/* The following line of code is equivalent to 
				( 1 * lx(1) ) + ( a(0) * d(0) ) since 
				lx(1) = lx(0) - d(0), so d(0) = lx(0) - lx(1)
			 The 1, 4, 5 is the number of years in each age bin
		*/
		gen nLx_`s' = 1*(lx_`s' - ndx_`s') + ax_`s'*1*ndx_`s' if _n==1	
		replace nLx_`s' = 4*(lx_`s' - ndx_`s') + ax_`s'*4*ndx_`s' if _n==2	
		replace nLx_`s' = 5*(lx_`s' - ndx_`s') + ax_`s'*5*ndx_`s' if _n>=3 & _n<=`penultimate'
		replace nLx_`s' = ndx_`s'/nMx_`s' if _n==`new_max_age_cat'	// terminal bin

		* Now continue to follow the derivation from part 1
		gen nTx_`s'=nLx_`s' if _n==`new_max_age_cat'
		forvalues i=1/`penultimate' {
			replace nTx_`s'=nLx_`s'[_n] + nTx_`s'[_n+1] if _n==`new_max_age_cat'-`i'
		}
		gen ex_`s'=nTx_`s'/lx_`s'
		
		* Prepare and store the life expectancy difference
		local c_abbrev = substr("`c'", 1, 5)
		local c_abbrev = subinstr("`c_abbrev'", "/", "", .)
		local c_abbrev = subinstr("`c_abbrev'", " ", "", .)
		* Here is the ounterfactual life expectancy at birth
		local ex_cf_birth_`s'_`c_abbrev' = ex_`s'[1] 
		local ex_diff_`s'_`c_abbrev' = round(`ex_cf_birth_`s'_`c_abbrev'' - `ex_birth_`s'', 0.001)
		* Due to issues with rounding errors, only use 3 digits after the decimal place
		di "The difference in life expectancy for `s's is `ex_diff_`s'_`c_abbrev''"
		assert `ex_diff_`s'_`c_abbrev'' >= 0
	} // end loop over birth sex
}


*** Part 3: Write results to file
/* As weights for aggregating across the sexes for a total life expectancy
	 difference, use sex ratio at birth: 1.073 M/F
   Source: World Bank https://data.worldbank.org/indicator/SP.POP.BRTH.MF
*/
putexcel set "`ROOT'/estimated_life_expectancy_differences_`region'.xlsx", modify
putexcel B2 = "Cause of death"
putexcel C2 = "Estimated life expectancy difference vs. actual: women"
putexcel D2 = "Estimated life expectancy difference vs. actual: men"
putexcel E2 = "Estimated life expectancy difference vs. actual: aggregate"
local iterator = 3
foreach c of local the_causes {
	local c_abbrev = substr("`c'", 1, 5)
	local c_abbrev = subinstr("`c_abbrev'", "/", "", .)
	local c_abbrev = subinstr("`c_abbrev'", " ", "", .)
	di "Cause is `c' and its abbreviation is `c_abbrev'"
	putexcel B`iterator' = "`c'"
	putexcel C`iterator' = `ex_diff_female_`c_abbrev''
	putexcel D`iterator' = `ex_diff_male_`c_abbrev''
	putexcel E`iterator' = ((1.073*`ex_diff_male_`c_abbrev''+`ex_diff_female_`c_abbrev'')/2.073)
	local iterator = `iterator' + 1
}
		

